/******************************************************************************
**                                                                           **
**    k4de - 3d-editor for the K Desktop Enviroment                          **
**                                                                           **
**    Copyright (C) 1999  Tobias Wollgam (tobias.wollgam@gmx.de)             **
**    Copyright (C) 1999  Markus Weber (mweber@gmx.de)                       **
**                                                                           **
**    This program is free software; you can redistribute it and/or modify   **
**    it under the terms of the GNU General Public License as published by   **
**    the Free Software Foundation; either version 2 of the License, or      **
**    (at your option) any later version.                                    **
**                                                                           **
**    This program is distributed in the hope that it will be useful,        **
**    but WITHOUT ANY WARRANTY; without even the implied warranty of         **
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          **
**    GNU General Public License for more details.                           **
**                                                                           **
**    You should have received a copy of the GNU General Public License      **
**    along with this program; if not, write to the Free Software            **
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              **
**                                                                           **
******************************************************************************/
/*
** slopemapdialog.cpp
*/
#include "slopemapdialog.h"
#include "slopemapdialog.moc"


#include "patternmodifierdialog.h"

#include <kapp.h>

SlopeMapWidget::SlopeMapWidget(QWidget *parent,slopeMap *s) :
	TextureWidgetBase(parent,"SlopeMapWidget")
{
	sm=s;

	pmd=new PatternModifierWidget(this,s->getMapModifier(),1 );
	connect(pmd,SIGNAL(valueChanged()),this,SIGNAL(valueChanged()) );

	tbasew=new TextureBaseWidget(this,sm);
	connect(tbasew,SIGNAL(valueChanged()),this,SIGNAL(valueChanged()) );

	paintEvent(NULL);
}

SlopeMapWidget::~SlopeMapWidget()
{
	if (tbasew) delete tbasew;
	if (pmd) delete pmd;
}

void SlopeMapWidget::getValue()
{
	if (pmd) pmd->getPatternModifier();
	if (tbasew) tbasew->getTextureBase();
}

QSize SlopeMapWidget::sizeHint() const
{
	return qs;
}

void SlopeMapWidget::paintEvent(QPaintEvent* qp)
{
int w=tbasew->sizeHint().width();
int h=tbasew->sizeHint().height();
int y=0,w1=0,h1=0;

	tbasew->setGeometry(10,10,w,h);
	y+=h+15;


	w1=pmd->sizeHint().width();
	h1=pmd->sizeHint().height();
        pmd->setGeometry(10,y,w1,h1);

	w=MAX(w1,w);
	h=y+h1+10;

	qs.setWidth(w+10);
	qs.setHeight(h+10);
	if (qp) QWidget::paintEvent(qp);
}

//------------------------------------------------------------------------

SlopeMapDialog::SlopeMapDialog(QWidget *parent,slopeMap *sm) :
	QDialog(parent,"SlopeMapDialog",true)
{
	setCaption(i18n("SlopeMap"));

	ok=new QPushButton(i18n("Ok") ,this);
	cancel=new QPushButton(i18n("Cancel"),this);

	pw=new SlopeMapWidget(this,sm);

	connect (ok,SIGNAL(pressed()),this,SLOT(okayPressed()) );
	connect (cancel,SIGNAL(pressed()),this,SLOT(cancelPressed()) );

	paintEvent(NULL);
}

SlopeMapDialog::~SlopeMapDialog()
{
	if (pw) delete pw;
	if (ok) delete ok;
	if (cancel) delete cancel;
}


void SlopeMapDialog::cancelPressed()
{
	QDialog::done(0);
}

void SlopeMapDialog::okayPressed()
{
	pw->getValue();
	QDialog::done(1);
}

void SlopeMapDialog::paintEvent(QPaintEvent* qp)
{
 	int w=pw->sizeHint().width();
	int h=pw->sizeHint().height();

	int w1=ok->sizeHint().width();
	int w2=cancel->sizeHint().width();
	int h1=ok->sizeHint().height();

	cancel->setGeometry(w-w2-5,h+5,w2,h1);
	ok->setGeometry(w-w2-w1-10,h+5,w1,h1);

	pw->setGeometry(0,0,w,h);

	setMinimumSize(w,h);
	setMaximumSize(w,h);

	if (qp) QDialog::paintEvent(qp);
}

